<?php
namespace App\Exports;

use App\ContestantInformation;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithMapping;
use Illuminate\Support\Facades\Storage;

class ContestantInformationExport implements FromCollection, WithHeadings, WithMapping
{
    protected $status;
    protected $country;
    protected $province;
    protected $vacancy_no;
    protected $gender;
    protected $degree;
    protected $overall_expereince;
    protected $relative_expereince;
    protected $full_time;
    protected $graduated;
    protected $exportDir;

    public function __construct($status,$country,$province, $vacancy_no,$gender,$degree,$overall_expereince,$relative_expereince,$full_time,$graduated, $exportDir)
    {
        $this->status = $status;
        $this->country = $country;
        $this->province = $province;
        $this->vacancy_no = $vacancy_no;
        $this->gender= $gender;
        $this->degree= $degree;
        $this->overall_expereince= $overall_expereince;
        $this->relative_expereince= $relative_expereince;
        $this->full_time= $full_time;
        $this->graduated= $graduated;
        $this->exportDir = $exportDir;
    }

    public function collection()
    {
        // if($this->status=="*"){
        //     return ContestantInformation::where('vacancy_no', $this->vacancy_no)
        //     ->get();
        // }
        // return ContestantInformation::where('status', $this->status)
        //     ->where('vacancy_no', $this->vacancy_no)
        //     ->get();
        
        return $contestants = ContestantInformation::query()

        ->when($this->status && $this->status !== '*', function($query) {
            $query->where('status', $this->status);
        })
        ->when($this->vacancy_no && $this->vacancy_no !== '*', function($query) {
            $query->where('vacancy_no', $this->vacancy_no);
        })
        ->when($this->country && $this->country !=='*', function($query) {
            $query->where('country', $this->country);
        })
        ->when($this->province && $this->province !=='*', function($query) {
            $query->where('province', $this->province);
        })
        ->when($this->gender && $this->gender !=='*', function($query) {
            $query->where('gender', $this->gender);
        })
        ->when($this->degree && $this->degree !=='*', function($query) {
            $query->whereHas('education', function($subQuery) {
                $subQuery->where('degree', $this->degree);
            });
        })
        ->when($this->graduated && $this->graduated !=='*', function($query) {
            $query->whereHas('education', function($subQuery) {
                $subQuery->where('graduated', $this->graduated);
            });
        })
        ->when($this->overall_expereince && $this->overall_expereince !=='*', function($query) {
            $query->whereHas('experience', function($subQuery) {
                $subQuery->where('years_of_experience', $this->overall_expereince);
            });
        })
        ->when($this->relative_expereince && $this->relative_expereince !=='*', function($query) {
            $query->whereHas('experience', function($subQuery) {
                $subQuery->where('years_of_relevant_experience', $this->relative_expereince);
            });
        })
        ->when($this->full_time && $this->full_time !=='*', function($query) {
            $query->whereHas('experience', function($subQuery) {
                $subQuery->where('full_time', $this->full_time);
            });
        })
        ->with(['experience', 'education', 'languages','skills'])
        ->get();
    }

    public function headings(): array
    {
        return [
            'ID', 'Full Name','Gender', 'Father Name','Email','Phone', 'Province', 'Status','Position Title', 'Vacancy No', 'CV', 'Certificates',
        ];
    }

    public function map($contestant): array
    {
        // Copy files to export directory and return relative path for Excel
        $cvPath = $this->copyFileToExportDirectory($contestant->cv);
        $certificatesPath = $this->copyFileToExportDirectory($contestant->certificates);
        $certificatesPath = str_replace('certificates/', '', $certificatesPath);
        $cvPath = str_replace('cv/', '', $cvPath);
        
        return [
            $contestant->id,
            $contestant->f_name.' '. $contestant->l_name,
            $contestant->gender,
            $contestant->father_name,
            $contestant->email,
            $contestant->phone,
            $contestant->province,
            $contestant->status,
            $contestant->description,
            $contestant->vacancy_no,
            $cvPath ? "=HYPERLINK(\"$cvPath\", \"CV\")" : '',
            // $cvPath->$cvPath,
            // $certificatesPath->$certificatesPath,
            $certificatesPath ? "=HYPERLINK(\"$certificatesPath\", \"Certificates\")" : '',
        ];
    }

    protected function copyFileToExportDirectory($fileName)
    {
        if (!$fileName) { 
            return '';
        }

        // Source path in storage
        $sourcePath = storage_path('app/' . $fileName);

        // Destination path in export directory
        $destinationPath = $this->exportDir . '/' . $fileName;

        // Copy file to export directory
        if (file_exists($sourcePath)) {
            copy($sourcePath, $destinationPath);
            return $fileName; // Return relative path for Excel
        }

        return '';
    }
}

?>