<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\ContestantInformation;
use App\Language;
use App\Education;
use App\Experience;
use App\Skill;
use App\Exports\ContestantInformationExport;
use Maatwebsite\Excel\Facades\Excel;
use ZipArchive;
use Illuminate\Support\Facades\Storage;
use DateTime;
use Carbon\Carbon;

class ContestantController extends Controller
{
    public function index(Request $request){
    

         // Validate the incoming data
         $validatedData = $request->validate([
            'f_name' => 'required|string|max:255',
            'l_name' => 'required|string|max:255',
            'father_name' => 'required|string|max:255',
            'gender' => 'required|string|max:255',
            'email' => 'required|email|max:255',
            'maritalstatus' => 'required|string|max:255',
            'country' => 'required|string|max:255',
            'province' => 'required|string|max:255',
            'district' => 'max:255',
            'permanent_province' => 'required|string|max:255',
            'permanent_district' => 'max:255',
            'dob' => 'required|date',
            'phone' => 'required|string|max:25',
            'whatsapp' => 'required|string|max:25',
            'user_consent' => 'required',
            'cv' => 'required|file|mimes:pdf,docx,xps||max:2048',
            'certificates' => 'required|file|mimes:pdf,docx,xps|max:2048',

        ]);


        // Check if the applicant already exists with the same email and vacancy_no
    $applicant = ContestantInformation::where('email', $request->email)
    ->where('vacancy_no', $request->vacancy_no) // Assuming vacancy_no is a column in your table
    ->first();

    if ($applicant) {
        // If the applicant exists, return a response indicating they have already applied
        return response()->json([
            'success' => false,
            'message' => 'You have already applied for this position.'
        ],422);
    }




        $contestant_information = new ContestantInformation;
        $contestant_information->f_name = $request->f_name;
        $contestant_information->l_name = $request->l_name;
        $contestant_information->father_name = $request->father_name;
        $contestant_information->gender = $request->gender;
        $contestant_information->email = $request->email;
        $contestant_information->marital_status = $request->maritalstatus;
        $contestant_information->country = $request->country;
        $contestant_information->province = $request->province;
        $contestant_information->district = $request->district;
        $contestant_information->permanent_province = $request->permanent_province;
        $contestant_information->permanent_district = $request->permanent_district;
        $contestant_information->dob = $request->dob;
        $contestant_information->phone = $request->phone;
        $contestant_information->whatsapp = $request->whatsapp;
        $contestant_information->vacancy_no = $request->vacancy_no;
        $contestant_information->description = $request->description;
        $contestant_information->user_consent = $request->user_consent=="on" ? true : false;

        $filecv = $request->file('cv');
        $fileNamecv =  $request->vacancy_no . time() . '_CV.' . $filecv->getClientOriginalExtension();
        $pathcv = $filecv->storeAs('documents/cv', $fileNamecv);

        $filecertificates = $request->file('certificates');
        $fileNamecertificates =  $request->vacancy_no .'_'. time() . '_Certificates.' . $filecertificates->getClientOriginalExtension();
        $pathcertificates = $filecertificates->storeAs('documents/certificates', $fileNamecertificates);

        $contestant_information->cv = $pathcv;
        $contestant_information->certificates = $pathcertificates;

        $contestant_information->save();


        // Assuming data comes as a JSON string like {"lang1": "English", "speak1": "low", ...}
        $data = $request->all();

        // We'll loop through the incoming data to insert them
        $languages = [];

        // Assuming a maximum of 10 language entries
        for ($i = 1; $i <= $request->language_count; $i++) {
            $langKey = "language{$i}";
            $speakKey = "speaking{$i}";
            $readKey = "reading{$i}";
            $writeKey = "writing{$i}";

            // Check if the language entry exists in the request
            if (isset($data[$langKey]) && isset($data[$speakKey]) && isset($data[$readKey]) && isset($data[$writeKey])) {
                $languages[] = [
                    'contestant_information_id' => $contestant_information->id,
                    'language' => $data[$langKey],
                    'speaking' => $data[$speakKey],
                    'reading' => $data[$readKey],
                    'writing' => $data[$writeKey],
                ];
            }
        } 

        // Insert all languages at once
        Language::insert($languages);

        $skills = [];

        // Assuming a maximum of 10 language entries
        for ($i = 1; $i <= $request->skill_count; $i++) {
            $skill = "skill{$i}";
            $skill_level = "skill_level{$i}";


            // Check if the language entry exists in the request
            if (isset($data[$skill_level]) && isset($data[$skill])) {
                $skills[] = [
                    'contestant_information_id' => $contestant_information->id,
                    'skill' => $data[$skill],
                    'skill_level' => $data[$skill_level],

                ];
            }
        }

        // Insert all languages at once
        Skill::insert($skills);



        // We'll loop through the incoming data to insert them
        $experience = [];

        // Assuming a maximum of 10 language entries
        for ($i = 1; $i <= $request->experience_count; $i++) {
            

            $employer_name_ = "employer_name_{$i}";
            $job_title_ = "job_title_{$i}";
            $start_date_ = "start_date_{$i}";
            $end_date_ = "end_date_{$i}";
            $currently_working_ = "currently_working_{$i}";
            $relative_experience_ = "relative_experience_{$i}";
            $experience_province_ = "experience_province_{$i}";
            $description_of_duties_ = "description_of_duties_{$i}";
            $full_time_ = "full_time_{$i}";
            $supervisor_name_ = "supervisor_name_{$i}";
            $supervisor_email_ = "supervisor_email_{$i}";
            $supervisor_phone_ = "supervisor_phone_{$i}";


            $from = Carbon::createFromFormat('Y-m-d', $data[$start_date_]);
            $to = Carbon::createFromFormat('Y-m-d', $data[$end_date_]);
            // Assuming the dates are in the correct format
            $start_date = new DateTime($from);
            $end_date = new DateTime($to);

            // Calculate total years of experience
            $years_of_experience = $end_date->diff($start_date)->y;

            // Initialize years_of_relevant_experience
            $years_of_relevant_experience = 0;

            if (isset($data[$relative_experience_]) == 'on') {
                // If relative experience is true, use the same start and end dates
                $years_of_relevant_experience = $years_of_experience;
                $years_of_experience = 0;
            }
            
            // Check if the language entry exists in the request
            if (isset($data[$employer_name_])) {
        
                $experience[] = [
                    'contestant_information_id' => $contestant_information->id,
                    'employer_name' => $data[$employer_name_],
                    'job_title' => $data[$job_title_],
                    'start_date' => $data[$start_date_],
                    'end_date' => $data[$end_date_],
                    'years_of_experience' => $years_of_experience,
                    'years_of_relevant_experience' => $years_of_relevant_experience,
                    'currently_working' => $data[$currently_working_] ?? "off",
                    'relative_experience' => $data[$relative_experience_] ?? "off",
                    'province' => $data[$experience_province_],
                    'description_of_duties' => $data[$description_of_duties_],
                    'full_time' => $data[$full_time_],
                    // 'type_of_busines' => $data[$type_of_business_],
                    'supervisor_name' => $data[$supervisor_name_],
                    'supervisor_email' => $data[$supervisor_email_],
                    'supervisor_phone' => $data[$supervisor_phone_],
                ];
            }
        }

        // Insert all languages at once
        Experience::insert($experience);


         // We'll loop through the incoming data to insert them
         $education = [];

         // Assuming a maximum of 10 language entries
         for ($i = 1; $i <= $request->education_count; $i++) {
             $degree = "degree{$i}";
             $institution = "institution{$i}";
             $edu_start_date = "edu_start_date{$i}";
             $edu_end_date = "edu_end_date{$i}";
             $graduated = "graduated{$i}";
             $education_level = "education_level{$i}";
             $edu_country = "edu_country{$i}";
             $edu_city = "edu_city{$i}";
          

 
             // Check if the language entry exists in the request
             if (isset($data[$langKey])) {
                 $education[] = [
                     'contestant_information_id' => $contestant_information->id,
                     'degree' => $data[$degree],
                     'institution_name' => $data[$institution],
                     'start_date' => $data[$edu_start_date],
                     'end_date' => $data[$edu_end_date],
                     'graduated' => $data[$graduated] ?? "off",
                     'education_level' => $data[$education_level],
                     'country' => $data[$edu_country],
                     'province' => $data[$edu_city],
                 ];
             }
         }
 
         // Insert all languages at once
         Education::insert($education);


         return response()->json([
            'success' => true,
            'message' => 'You have successfully applied for this position.'
        ],200);
    
    }
    public function adminIndex(){
        $contestant_information = ContestantInformation::all();
        return view('applicants.applicants',['applicants'=>$contestant_information]);
    }
    public function show($id)
    {
        $applicant = ContestantInformation::with(['languages', 'education', 'experience','skills'])->findOrFail($id);
        // Get the next applicant ID
        $nextApplicantId = ContestantInformation::with(['languages', 'education', 'experience','skills'])->findOrFail($id)->where('id', '>', $id)->min('id');

        // Get the previous applicant ID
        $previousApplicantId = ContestantInformation::where('id', '<', $id)->max('id');

        $filePath = storage_path('app/'.$applicant->cv); // Update with your file path
        $fileExtension = pathinfo($filePath, PATHINFO_EXTENSION);
        $fileType='';
        if ($fileExtension === 'pdf') {
            $fileType ="pdf";
        } elseif ($fileExtension === 'docx') {
            $fileType ="docx";
            
        } elseif ($fileExtension === 'xps') {
            $fileType ="xps";
            
        } else {
            $fileType ="*";
        }
        


        return view('applicants.show', compact('fileType','applicant', 'nextApplicantId', 'previousApplicantId'));
    }
    public function updateStatus(Request $request, $id)
    {
        $request->validate([
            'status' => 'required|string',
        ]);
    
        $applicant = ContestantInformation::findOrFail($id);
        $applicant->status = $request->status;
        $applicant->save();
    
        return response()->json(['message' => 'Status updated successfully!']);
    }
    // ApplicantController.php
    public function viewFile($id, $type)
    {
        $applicant = ContestantInformation::findOrFail($id);

        // Assuming you have stored the file path in the database
        $filePath = storage_path('app/'.$applicant->cv);
        $filePath1 = $applicant->cv;

        if (!file_exists($filePath)) {
            abort(404, 'File not found');
        }

        // For DOCX or XPS, redirect to a Google Docs viewer or another appropriate viewer
        if ($type === 'docx' || $type === 'xps') {
            return view('applicants.view-doc', compact('filePath','filePath1'));
        }

        // For PDF, return the PDF view directly
        return response()->file($filePath);
    }
    public function viewFileCert($id, $type)
    {
        $applicant = ContestantInformation::findOrFail($id);

        // Assuming you have stored the file path in the database
        $filePath = storage_path('app/'.$applicant->certificates);
        
        if (!file_exists($filePath)) {
            abort(404, 'File not found');
        }

        // For DOCX or XPS, redirect to a Google Docs viewer or another appropriate viewer
        if ($type === 'docx' || $type === 'xps') {
            return view('applicants.view-doc', compact('filePath'));
        }

        // For PDF, return the PDF view directly
        return response()->file($filePath);
    }

    public function export(Request $request)
    {
        $status = $request->status;
        $vacancy_no = $request->vacancy_no;
        $country = $request->country;
        $province = $request->province;
        $gender = $request->gender;
        $degree = $request->degree;
        $overall_expereince = $request->overall_experience;
        $relative_expereince = $request->relevant_experience;
        $full_time = $request->full_time;
        $graduated = $request->graduated;

       
        
        // Temporary directory for export
        $exportDir = storage_path('app');
        $documentsDir = $exportDir . '/documents';
        
        // Create export directory if it doesn't exist
        if (!file_exists($exportDir)) {
            mkdir($exportDir, 0777, true);
        }
        
        if (!file_exists($documentsDir)) {
            mkdir($documentsDir, 0777, true);
        }
    
        // Export to Excel
        $excelFilePath = 'public/export/applicants.xlsx';
        Excel::store(new ContestantInformationExport($status,$country,$province, $vacancy_no,$gender,$degree,$request->overall_experience,$request->relevant_experience,$full_time,$graduated, $exportDir), $excelFilePath);
    
        // if($status=='*'){
        //     $contestants = ContestantInformation::where('vacancy_no', $vacancy_no)
        //     ->get();   
        // }


        // Copy related files to the export directory
        // if($vacancy_no !='' && $vacancy_no !='*')
        // {
        //     $contestants = ContestantInformation::where('vacancy_no', $vacancy_no)
        //                                     ->get();
        // }
        $contestants = ContestantInformation::query()
        // // Apply filters from frontend
        // ->when($request->status && $request->status !== '*', function($query) use ($request) {
        //     $query->where('status', $request->status);
        // })
        ->when($request->status && $request->status !== '*', function($query) use ($request) {
            $query->where('status', $request->status);
        })
        ->when($request->vacancy_no && $request->vacancy_no !== '*', function($query) use ($request) {
            $query->where('vacancy_no', $request->vacancy_no);
        })
        ->when($request->country && $request->country !=='*', function($query) use ($request) {
            $query->where('country', $request->country);
        })
        ->when($request->province && $request->province !=='*', function($query) use ($request) {
            $query->where('province', $request->province);
        })
        ->when($request->gender && $request->gender !=='*', function($query) use ($request){
            $query->where('gender', $request->gender);
        })
        ->when($request->degree && $request->degree !=='*', function($query) use ($request) {
            $query->whereHas('education', function($subQuery) use ($request){
                $subQuery->where('degree', $request->degree);
            });
        })
        ->when($request->graduated && $request->graduated !=='*', function($query) use ($request) {
            $query->whereHas('education', function($subQuery) use ($request) {
                $subQuery->where('graduated', $request->graduated);
            });
        })
        ->when($request->overall_experience && $request->overall_experience !=='*', function($query) use ($request){
            $query->whereHas('experience', function($subQuery) use ($request) {
                $subQuery->where('years_of_experience', $request->overall_experience);
            });
        })
        ->when($request->relevant_experience && $request->relevant_experience !=='*', function($query) use ($request){
            $query->whereHas('experience', function($subQuery) use ($request){
                $subQuery->where('years_of_relevant_experience', $request->relevant_experience);
            });
        })
        ->when($request->full_time && $request->full_time !=='*', function($query) use ($request){
            $query->whereHas('experience', function($subQuery) use ($request){
                $subQuery->where('full_time', $request->full_time);
            });
        })
        // // Fetch with relationships
        ->with(['experience', 'education', 'languages','skills'])
        ->get();

        // $contestants = ContestantInformation::where('vacancy_no', ($vacancy_no=='*' || $vacancy_no=='') ? 'vacancy_no' : $vacancy_no)
        //                                     ->where('country', $country)
        //                                     ->where('province', $province)
        //                                     ->where('gender', $gender)
        //                                     ->where('marital_status', $maritalstatus)
        //                                     ->get();
                                        
        // return $contestants;
        if(!$contestants->isEmpty()){
            foreach ($contestants as $contestant) {
                if ($contestant->cv) {
                    $fileName = basename($contestant->cv);
                    $destinationPath = 'public/export/documents/' . $fileName;
        
                    // Ensure unique file name
                    while (Storage::exists($destinationPath)) {
                        $fileName = pathinfo($fileName, PATHINFO_FILENAME) . '_' . time() . '.' . pathinfo($fileName, PATHINFO_EXTENSION);
                        $destinationPath = 'public/export/documents/' . $fileName;
                    }
        
                    Storage::copy($contestant->cv, $destinationPath);
                }
        
                if ($contestant->certificates) {
                    $fileName = basename($contestant->certificates);
                    $destinationPath = 'public/export/documents/' . $fileName;
        
                    // Ensure unique file name
                    while (Storage::exists($destinationPath)) {
                        $fileName = pathinfo($fileName, PATHINFO_FILENAME) . '_' . time() . '.' . pathinfo($fileName, PATHINFO_EXTENSION);
                        $destinationPath = 'public/export/documents/' . $fileName;
                    }
        
                    Storage::copy($contestant->certificates, $destinationPath);
                }
            }
        
            // Create zip archive
            $zipPath = storage_path('app/public/applicants_export.zip');
            $zip = new ZipArchive();
        
            if ($zip->open($zipPath, ZipArchive::CREATE | ZipArchive::OVERWRITE) === TRUE) {
                // Add Excel file to zip
                $zip->addFile(storage_path('app/' . $excelFilePath), 'applicants.xlsx');
        
                // Add documents to zip
                $files = scandir(storage_path('app/public/export/documents/'));
                // return $files;
    
                foreach ($files as $file) {
                    // echo $file;
                    // Ignore . and .. directories
                    if ($file === '.' || $file === '..') {
                        continue;
                    }
                    // return is_file(storage_path('app/public/export/documents') . '/' . $file);
                    // Check if it's a file and not a directory
                    if (is_file(storage_path('app/public/export/documents') . '/' . $file)) {
    
                        // return storage_path('app/public/export/documents') . '/' . $file;
                        // Copy the file to the documents directory within the ZIP
                        $zip->addFile(storage_path('app/public/export/documents') . '/' . $file, 'documents/' . $file);
                    }
                }
                $zip->close();
            }
    
        
            // Clean up the temporary files
            array_map('unlink', glob("app/public/export/*"));
            function deleteDirectory($dir) {
                if (!file_exists($dir)) {
                    return true;
                }
            
                if (!is_dir($dir)) {
                    return unlink($dir);
                }
            
                $items = scandir($dir);
                foreach ($items as $item) {
                    if ($item === '.' || $item === '..') {
                        continue;
                    }
            
                    $path = $dir . DIRECTORY_SEPARATOR . $item;
                    if (is_dir($path)) {
                        deleteDirectory($path);
                    } else {
                        unlink($path);
                    }
                }
            
                return rmdir($dir);
            }
            deleteDirectory(storage_path('app/public/export'));
            
        
            // Return the zip file for download
            // return response()->json(['message' => 'File is ready for download', 'download_url' => asset('storage/contestants_export.zip')]);
            return response()->download('storage/applicants_export.zip');
        }
        return 'No files to export';
       
    }
    public function deleteSelected(Request $request)
    {
        $status = $request->status;
        $vacancy_no = $request->vacancy_no;
        $country = $request->country;
        $province = $request->province;
        $gender = $request->gender;
        $maritalstatus = $request->maritalstatus;
        $degree = $request->degree;
        $type_of_business = $request->type_of_business;

        // Fetch contestants based on the filters provided
        $contestants = ContestantInformation::query()
            ->when($request->status && $request->status !== '*', function($query) use ($request) {
                $query->where('status', $request->status);
            })
            ->when($request->vacancy_no && $request->vacancy_no !== '*', function($query) use ($request) {
                $query->where('vacancy_no', $request->vacancy_no);
            })
            ->when($request->country && $request->country !== '*', function($query) use ($request) {
                $query->where('country', $request->country);
            })
            ->when($request->province && $request->province !== '*', function($query) use ($request) {
                $query->where('province', $request->province);
            })
            ->when($request->gender && $request->gender !== '*', function($query) use ($request) {
                $query->where('gender', $request->gender);
            })
            ->when($request->maritalstatus && $request->maritalstatus !== '*', function($query) use ($request) {
                $query->where('marital_status', $request->maritalstatus);
            })
            ->when($request->degree && $request->degree !== '*', function($query) use ($request) {
                $query->whereHas('education', function($subQuery) use ($request) {
                    $subQuery->where('degree', $request->degree);
                });
            })
            ->when($request->type_of_business && $request->type_of_business !== '*', function($query) use ($request) {
                $query->whereHas('experience', function($subQuery) use ($request) {
                    $subQuery->where('type_of_business', $request->type_of_business);
                });
            })
            ->get();

        // Loop through each contestant to delete files and records
        foreach ($contestants as $contestant) {
            // Delete the CV file if it exists
            if ($contestant->cv) {
                Storage::delete($contestant->cv);
            }

            // Delete the certificates file if it exists
            if ($contestant->certificates) {
                Storage::delete($contestant->certificates);
            }

            // Delete the contestant record from the database
            $contestant->delete();
        }

        return redirect('/applicants');
    }

}
