<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Slideshow;
use App\About;
use App\Post;
use App\Address;
use App\CareerOpportunity;
use App\CareerDetail;
use App\Helpers\TextHelper;
use App\BnfOverview;


class LandingPageController extends Controller
{
    public function maintenance(){
        return view('maintenance');
    }
    public function index(){
        $slideshow = Slideshow::all();
        $about = About::first();
        // $post = Post::all();
        $address = Address::first();

        $categories = Post::select('category')->distinct()->get();
        
        $posts = collect();
        
        foreach ($categories as $category) {
            $categoryPosts = Post::where('category', $category->category)
                ->limit(6)
                ->get()
                ->map(function ($item) {
                    $item->truncated_description = TextHelper::truncateByLines($item->description, 3);
                    return $item;
                });
            $posts->push($categoryPosts);
        }

        // $postexp = Post::all()->map(function ($item) {
        //     $item->truncated_description = TextHelper::truncateByLines($item->description, 8);
        //     return $item;
        // });
        // $categories = Post::select('category')->distinct()->get();
        
        $postsexp = collect();
        
        foreach ($categories as $category) {
            $categoryPosts = Post::where('category', $category->category)
                ->limit(6)
                ->get()
                ->map(function ($item) {
                    $item->truncated_description = TextHelper::truncateByLines($item->description, 5);
                    return $item;
                });
            $postsexp->push($categoryPosts);
        }
        // return $posts;

        // return $postsexp[0];


        // return $posts;

        return view('welcome',['slideshows'=>$slideshow,'about'=>$about,'posts'=>$posts,'address'=>$address,'postexp'=>$postsexp]);
    }
    public function fetchData()
    {
        $bnfOverview = BnfOverview::first();
        return response()->json($bnfOverview);
    }
    public function search(Request $request){

        // $post = Post::where('title','LIKE','%'.$request->search.'%')->orWhere('description','LIKE','%'.$request->search.'%')->orWhere('location','LIKE','%'.$request->search.'%')->get();
        $post = Post::where('title','LIKE','%'.$request->search.'%')->orWhere('description','LIKE','%'.$request->search.'%')->orWhere('location','LIKE','%'.$request->search.'%')->get()->map(function ($item) {
            $item->truncated_description = TextHelper::truncateByLines($item->description, 5);
            return $item;
        });
        $slideshow = Slideshow::all();
        $address = Address::first();
        // return $post;
        return view('search',['posts'=> $post,'slideshows'=>$slideshow,'address'=>$address]);
    }
    public function career(){
        $career = CareerOpportunity::all();
        $slideshow = Slideshow::all();
        $address = Address::first();

        $locations = CareerOpportunity::select('location')->distinct()->get()->pluck('location');
        $positions = CareerOpportunity::select('title')->distinct()->get()->pluck('title');
        $careerOpportunities = CareerOpportunity::all();

        return view('career',['careers'=>$career,'slideshows'=>$slideshow,'address'=>$address,'locations'=>$locations,'positions'=>$positions,'careerOpportunities'=>$careerOpportunities]);
    }
    public function procurement(){
        $career = CareerOpportunity::all();
        $slideshow = Slideshow::all();
        $address = Address::first();

        $locations = CareerOpportunity::select('location')->distinct()->get()->pluck('location');
        $positions = CareerOpportunity::select('title')->distinct()->get()->pluck('title');
        $careerOpportunities = CareerOpportunity::where('career_type','procurement')->get();

        return view('procurement',['careers'=>$career,'slideshows'=>$slideshow,'address'=>$address,'locations'=>$locations,'positions'=>$positions,'careerOpportunities'=>$careerOpportunities]);
    }
    public function career_apply($id){
        $career = CareerOpportunity::find($id);
        return view('career_apply',['career'=>$career]);

    }
    public function career_details($id){
        $careerDetails = CareerDetail::where('career_opportunity_id', $id)->get();
        $career = CareerOpportunity::find($id);

        
        return view('career_details',['career'=>$career,'careerDetails'=>$careerDetails]);
    }
    public function like(Request $request){
        $item = Post::find($request->id);

        if ($item) {
            $item->liked = $item->liked ? 0 : 1; // Toggle liked status
            $item->save();

            return response()->json(['success' => true, 'liked' => $item->liked]);
        }

        return response()->json(['success' => false]);
    }
    public function view_all_posts(Request $request){
        // $posts = Post::where('category',$request->post_type)->get();

        $posts = Post::where('category',$request->post_type)->get()->map(function ($item) {
            $item->truncated_description = TextHelper::truncateByLines($item->description, 5);
            return $item;
        });
        $postexp = Post::all()->map(function ($item) {
            $item->truncated_description = TextHelper::truncateByLines($item->description, 8);
            return $item;
        });


        $slideshow = Slideshow::all();
        $address = Address::first();

        return view('view_all_posts',['post_type'=>$request->post_type,'postexp'=>$postexp,'posts'=>$posts,'slideshows'=>$slideshow,'address'=>$address]);
    }
}
