<?php

namespace App\Http\Controllers;

use App\Post;
use Illuminate\Http\Request;

class PostController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
    }
    public function index()
    {
        $posts = Post::all();
        return view('posts.index', compact('posts'));
    }

    public function create()
    {
        return view('posts.create');
    }
    public function store(Request $request)
    {
        $request->validate([
            'category' => 'required',
            'title' => 'required',
            'date' => 'required',
            'location' => 'required',
            'description' => 'required',
            'image' => 'required|image|mimes:jpeg,png,jpg,gif,svg|max:2048',
        ]);
    
        $imageName = time().'.'.$request->image->extension();
        $request->image->move(public_path('images'), $imageName);
    
        $post = new Post;
        $post->category = $request->category;
        $post->title = $request->title;
        $post->date = $request->date;
        $post->location = $request->location;
        $post->description = $request->description;
        $post->image = $imageName;
        $post->liked = $request->liked;
        $post->save();

        return redirect()->route('posts.index')
                        ->with('success','Post created successfully.');
    }
    
    public function update(Request $request, Post $post)
    {
       
        $request->validate([
            'category' => 'required',
            'title' => 'required',
            'date' => 'required',
            'location' => 'required',
            'description' => 'required',
        ]);
    
        if($request->hasFile('image')){
            $imageName = time().'.'.$request->image->extension();
            $request->image->move(public_path('images'), $imageName);
            $post->image = $imageName;
        }

        $post = Post::find($post->id);
        $post->category = $request->category;
        $post->title = $request->title;
        $post->date = $request->date;
        $post->location = $request->location;
        $post->description = $request->description;
        $post->liked = $request->liked;
        $post->save();
    
        return redirect()->route('posts.index')
                        ->with('success','Post updated successfully');
    }
   

    public function show(Post $post)
    {
        return view('posts.show',compact('post'));
    }

    public function edit(Post $post)
    {
        return view('posts.edit',compact('post'));
    }

  
    public function destroy(Post $post)
    {
        $post->delete();

        return redirect()->route('posts.index')
                        ->with('success','Post deleted successfully');
    }
    public function uploadImage(Request $request)
    {
        if($request->hasFile('image')) {
            $image = $request->file('image');
            $imageName = time().'.'.$image->getClientOriginalExtension();
            $image->move(public_path('images'), $imageName);
            $imagePath = url('images/'.$imageName);
            return response()->json(['url' => $imagePath]);
        }

        return response()->json(['error' => 'No image uploaded'], 400);
    }
}
