<!DOCTYPE html>
<html>
<head>
    <title>BNF Overview</title>
    <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/4.0.0/css/bootstrap.min.css">
    <script src="https://code.jquery.com/jquery-3.6.0.min.js"></script>
    <script src="https://cdn.jsdelivr.net/npm/chart.js"></script>
</head>
<body>
<div class="container mt-5">
    <div class="card mt-3">
        <div class="card-header">
            BNF Overview Data
        </div>
        <div class="card-body">
            <table class="table">
                <thead>
                    <tr>
                        <th>Male</th>
                        <th>Female</th>
                        <th>CU5</th>
                        <th>PWD</th>
                        <th>PLW</th>
                        <th>Actions</th>
                    </tr>
                </thead>
                <tbody id="bnfTableBody">
                    <!-- Data will be appended here via AJAX -->
                </tbody>
            </table>
        </div>
    </div>
    <div class="card mt-3">
        <div class="card-header">
            Add BNF Overview Data
        </div>
        <div class="card-body">
            <form id="bnfForm">
                <?php echo csrf_field(); ?>
                <div class="form-group">
                    <label for="male">Male</label>
                    <input type="number" class="form-control" id="male" name="male">
                </div>
                <div class="form-group">
                    <label for="female">Female</label>
                    <input type="number" class="form-control" id="female" name="female">
                </div>
                <div class="form-group">
                    <label for="CU5">CU5</label>
                    <input type="number" class="form-control" id="CU5" name="CU5">
                </div>
                <div class="form-group">
                    <label for="PWD">PWD</label>
                    <input type="number" class="form-control" id="PWD" name="PWD">
                </div>
                <div class="form-group">
                    <label for="PLW">PLW</label>
                    <input type="number" class="form-control" id="PLW" name="PLW">
                </div>
                <button type="submit" class="btn btn-primary">Add</button>
            </form>
        </div>
    </div>


    <div class="card">
        <div class="card-header">
            BNF Overview
        </div>
        <div class="card-body">
            <div class="row">
                <div class="col-sm-6">
                    <canvas id="myPieChart"></canvas>
                </div>
            </div>
        </div>
    </div>

  
</div>

<script>
$(document).ready(function() {
    function fetchData() {
        $.ajax({
            url: '/bnf_overviewfetch',
            method: 'GET',
            success: function(data) {
                const ctx = document.getElementById('myPieChart').getContext('2d');
                const total = data.male + data.female + data.CU5 + data.PWD + data.PLW;
                const percentages = [
                    ((data.female / total) * 100).toFixed(2),
                    ((data.male / total) * 100).toFixed(2),
                    ((data.CU5 / total) * 100).toFixed(2),
                    ((data.PWD / total) * 100).toFixed(2),
                    ((data.PLW / total) * 100).toFixed(2)
                ];
                const chartData = {
                    labels: ['Female', 'Male', 'CU5', 'PWD', 'PLW'],
                    datasets: [{
                        data: [data.female, data.male, data.CU5, data.PWD, data.PLW],
                        backgroundColor: [
                            'rgba(255, 40, 145, 0.5)',
                            'rgba(0, 10, 255, 0.5)',
                            'rgba(248, 255, 60, 0.5)',
                            'rgba(255, 0, 29, 0.5)',
                            'rgba(10, 245, 9, 0.5)',
                        ],
                        borderColor: [
                            'rgba(255, 99, 132, 1)',
                            'rgba(54, 162, 235, 1)',
                            'rgba(255, 206, 86, 1)',
                            'rgba(75, 192, 192, 1)',
                        ],
                        borderWidth: 1
                    }]
                };

                new Chart(ctx, {
                    type: 'pie',
                    data: chartData,
                    options: {
                        responsive: true,
                        plugins: {
                            legend: {
                                position: 'top',
                            },
                            tooltip: {
                                callbacks: {
                                    label: function(tooltipItem) {
                                        const value = chartData.datasets[0].data[tooltipItem.dataIndex];
                                        const percentage = percentages[tooltipItem.dataIndex];
                                        return `${value} (${percentage}%)`;
                                    }
                                }
                            }
                        }
                    }
                });

                const tableBody = $('#bnfTableBody');
                tableBody.empty();
                tableBody.append(`
                    <tr>
                        <td>${data.male}</td>
                        <td>${data.female}</td>
                        <td>${data.CU5}</td>
                        <td>${data.PWD}</td>
                        <td>${data.PLW}</td>
                        <td>
                            <a href="/bnf_overview/${data.id}" class="btn btn-primary btn-edit" data-id="${data.id}">Edit</button>
                         
                        </td>
                    </tr>
                `);
            }
        });
    }

    fetchData();

    $('#bnfForm').on('submit', function(e) {
        e.preventDefault();
        $.ajax({
            url: '/bnf_overview',
            method: 'POST',
            data: $(this).serialize(),
            success: function(response) {
                if (response.error) {
                alert(response.error);
                    
                }else{
                    alert(response.success);
                    fetchData();
                }
                
            }
        });
    });

    $(document).on('click', '.btn-delete', function() {
        const id = $(this).data('id');
        if (confirm('Are you sure you want to delete this record?')) {
            $.ajax({
                url: '/bnf_overview/' + id,
                method: 'GET',
                success: function(response) {
                    alert(response.success);
                    fetchData();
                }
            });
        }
    });
});
</script>
</body>
</html>
<?php /**PATH D:\Softwares\Web Applcations\ASIO WebPage\ASIO_web_page\asio_web\resources\views/bnf_overview/index.blade.php ENDPATH**/ ?>